PROGRAM driver1
USE Cubic_Shepard
IMPLICIT NONE
 
! Code converted using TO_F90 by Alan Miller
! Date: 1999-08-10  Time: 20:55:48
 
!                          CS2TST
!                         11/20/98

!   This program computes interpolation errors using the scattered data
! package CSHEP2D for each of ten test functions and a 33 by 33 uniform grid
! of interpolation points in the unit square.

!   This program uses Subroutines TESTDT and TSTFN1 from ACM Algorithm SURVEY
! to generate a node set and and the test function values.

INTEGER, PARAMETER :: nmax=100, nrmax=10, ni=33

! Array storage:

REAL (dp) :: x(nmax), y(nmax), w(nmax), rw(nmax), a(9,nmax), p(ni), ft(ni,ni)
INTEGER   :: lcell(nrmax,nrmax), lnext(nmax)

REAL (dp) :: del, dum, dx, dy, ermax, ermean, pw,  &
             rmax, ssa, sse, ssm, sum, xmin, ymin
INTEGER   :: i, ier, j, k, kf, kff, kfl, ks, n, nc, np, nr, nw, nwmax

! Data:

! LOUT = Logical unit number for output.
! NSET = Number of node sets.
! NFUN = Number of test functions.

INTEGER , PARAMETER  ::  lout = 2, nset = 5
INTEGER   :: nfun = 10

OPEN (lout, FILE='res1_2')

! Input format:

100 FORMAT (i2)

! Get a user-specified node set number KS.

1 WRITE (*,110) nset
110 FORMAT (//t14, 'CS2TST:  CSHEP2D Test Program'//  &
            t6, 'Specify a node set number in the range 1', ' to ', i2, ':'/)
READ (*,100, ERR=1) ks
IF (ks < 1  .OR.  ks > nset) GO TO 1

! Copy N and the nodal coordinates for node set KS.

CALL testdt (ks, n, x, y)
IF (n < 10  .OR.  n > nmax) GO TO 20

! Allow the user to specify a range of function numbers.

WRITE (*,120) nfun
120 FORMAT (//'     Specify the first test function (1 to ',i2,'):')
READ (*,100,ERR=1) kff
IF (kff < 1  .OR.  kff > nfun) GO TO 1

WRITE (*,130) kff, nfun
130 FORMAT (//'     Specify the last test function (', i2,' to ',i2,'):'/)
READ (*,100,ERR=1) kfl
IF (kfl < kff  .OR.  kfl > nfun) GO TO 1
nfun = kfl-kff+1

! Input NC, NW, and NR from the console.

nwmax = MIN(40,n-1)
2 WRITE (*,140) n
140 FORMAT (//'     N =', i4//         &
            '     Specify the number of nodes NC for the least squares fits.'/ &
            '     NC = 17 is recommended.  NC >= 9.'//  &
            '          NC = ? '/)
READ (*, 100, ERR=2) nc
IF (nc < 9  .OR.  nc > nwmax) GO TO 2

3 WRITE (*,150)
150 FORMAT (//'     Specify the number of nodes NW for the weights.  NW = 30 is'/ &
            '     recommended.  1 <= NW <= MIN(40,N-1).'// &
            '          NW = ? '/)
READ (*,100,ERR=2) nw
IF (1 > nw  .OR.  nw > nwmax) GO TO 3

4 WRITE (*,160) nrmax
160 FORMAT (//'     Specify the number of rows and columns NR in the uniform grid'/ &
            '     of cells used to locate nearest neighbors.  NR = Sqrt(N/3) is'/ &
            '     recommended.  1 <= NR <= ', i2, '.'// &
            '          NR = ? '/)
READ (*,100,ERR=3) nr
IF (nr < 1  .OR.  nr > nrmax) GO TO 4

! Set up uniform grid points.

del = 1./DBLE(ni-1)
DO  i = 1,ni
  p(i) = DBLE(i-1)*del
END DO

! Initialize the average SSE/SSM value to zero.

ssa = 0.

! Print a heading and loop on test functions.

WRITE (lout,200) ks, n, ni, nc, nw, nr
DO  kf = kff,kfl
  
!   Compute true function values at the nodes.
  
  DO  k = 1,n
    CALL tstfn1 (kf, x(k), y(k), 0, w(k), dum, dum)
  END DO
  
!   Compute true function values FT on the uniform grid, and accumulate
!     the sum of values SUM and sum of squared values SSM.
  
  sum = 0.
  ssm = 0.
  DO  i = 1,ni
    DO  j = 1,ni
      CALL tstfn1 (kf, p(i), p(j), 0, ft(i,j), dum, dum)
      sum = sum + ft(i,j)
      ssm = ssm + ft(i,j)**2
    END DO
  END DO
  
!   Compute the sum of squared deviations from the mean SSM.
  
  ssm = ssm - sum*sum/DBLE(ni*ni)
  
!   Compute parameters A and RW defining the interpolant.
  
  CALL cshep2 (n, x, y, w, nc, nw, nr, lcell, lnext, xmin, ymin, dx, dy,  &
               rmax, rw, a, ier)
  IF (ier /= 0) GO TO 21
  
!   Compute interpolation errors.
  
  ermean = 0.
  ermax = 0.
  sse = 0.
  DO  i = 1,ni
    DO  j = 1,ni
      pw = cs2val (p(i), p(j), n, x, y, w, nr, lcell, lnext,  &
                   xmin, ymin, dx, dy, rmax, rw, a) - ft(i,j)
      ermean = ermean + ABS(pw)
      ermax = MAX(ermax,ABS(pw))
      sse = sse + pw*pw
    END DO
  END DO
  np = ni*ni
  ermean = ermean / np
  sse = sse/ssm
  ssa = ssa + sse
  WRITE (lout,210) kf, ermax, ermean, sse
END DO

! Print the average SSE/SSM value (averaged over the test functions).

WRITE (lout,220) ssa / nfun
STOP

! N is outside its valid range.

20 WRITE (lout,500) n, nmax
STOP

! Error in CSHEP2.

21 IF (ier == 2) WRITE (lout,510)
IF (ier == 3) WRITE (lout,520)
STOP

! Print formats:

200 FORMAT (t31, 'CS2TST Output:'//  &
            t18, 'CSHEP2D Interpolation Errors for N nodes and an'/  &
            t8, 'NI by NI Uniform Grid of Interpolation Points in the Unit Square'// &
            t8, 'Node set ', i2, '    N =', i4, '    NI = ',i2,  &
            '    NC = ', i2, '    NW = ', i2, '    NR = ',i2///  &
            t18, 'Function', '    Max Error', '    Mean Error    SSE/SSM'/)
210 FORMAT (t21, i2, t32, f7.4, t45, f8.5, '  ', f9.6)
220 FORMAT (//t13, 'Average SSE/SSM over the test functions = ', f9.6)

! Error message formats:

500 FORMAT (//t12, '*** Error in data -- N = ', i4, ', Maximum value =',  &
            i4, ' ***')
510 FORMAT (//t16, '*** Error in CSHEP2 -- duplicate nodes encountered ***')
520 FORMAT (///t16, '*** Error in CSHEP2 -- all nodes are collinear ***')


CONTAINS


SUBROUTINE testdt (k, n, x, y)

! Code converted using TO_F90 by Alan Miller
! Date: 1999-08-10  Time: 20:56:04

INTEGER, INTENT(IN)     :: k
INTEGER, INTENT(OUT)    :: n
REAL (dp), INTENT(OUT)  :: x(100)
REAL (dp), INTENT(OUT)  :: y(100)

!***********************************************************

!                                            Robert J. Renka
!                                  Dept. of Computer Science
!                                       Univ. of North Texas
!                                           renka@cs.unt.edu
!                                                   03/28/97

!   This subroutine returns one of five sets of nodes used
! for testing scattered data fitting methods.  All five sets
! approximately cover the unit square [0,1] X [0,1]:  the
! convex hulls of sets 1 and 3 extend slightly outside the
! square but do not completely cover it, those of sets 2 and
! 5 coincide with the unit square, and the convex hull of
! set 4 is a large subset of the unit square.

! On input:

!       K = Integer in the range 1 to 5 which determines the
!           choice of data set as follows:

!               K = 1 - Franke's 100-node set
!               K = 2 - Franke's 33-node set
!               K = 3 - Lawson's 25-node set
!               K = 4 - Random 100-node set
!               K = 5 - Gridded 81-node set

!       X,Y = Arrays of length at least N(K), where
!             N(1) = 100, N(2) = 33, N(3) = 25,
!             N(4) = 100, and N(5) = 81.

! Input parameters are not altered by this routine.

! On output:

!       N = Number of nodes in set K, or 0 if K is outside its valid range.

!       X,Y = Nodal coordinates of node set K.

! Subprograms required by TESTDT:  None

!***********************************************************

INTEGER   :: i

! Node set 1:  Franke's 100-node set.

REAL (dp), PARAMETER  :: x1(1:100) = (/   &
    0.0227035,  0.0539888, 0.0217008,  0.0175129, &
    0.0019029, -0.0509685, 0.0395408, -0.0487061, &
    0.0315828, -0.0418785, 0.1324189,  0.1090271, &
    0.1254439,  0.0934540, 0.0767578,  0.1451874, &
    0.0626494,  0.1452734, 0.0958668,  0.0695559, &
    0.2645602,  0.2391645, 0.2088990,  0.2767329, &
    0.1714726,  0.2266781, 0.1909212,  0.1867647, &
    0.2304634,  0.2426219, 0.3663168,  0.3857662, &
    0.3832392,  0.3179087, 0.3466321,  0.3776591, &
    0.3873159,  0.3812917, 0.3795364,  0.2803515, &
    0.4149771,  0.4277679, 0.4200010,  0.4663631, &
    0.4855658,  0.4092026, 0.4792578,  0.4812279, &
    0.3977761,  0.4027321, 0.5848691,  0.5730076, &
    0.6063893,  0.5013894, 0.5741311,  0.6106955, &
    0.5990105,  0.5380621, 0.6096967,  0.5026188, &
    0.6616928,  0.6427836, 0.6396475,  0.6703963, &
    0.7001181,  0.6333590, 0.6908947,  0.6895638, &
    0.6718889,  0.6837675, 0.7736939,  0.7635332, &
    0.7410424,  0.8258981, 0.7306034,  0.8086609, &
    0.8214531,  0.7290640, 0.8076643,  0.8170951, &
    0.8424572,  0.8684053, 0.8366923,  0.9418461, &
    0.8478122,  0.8599583, 0.9175700,  0.8596328, &
    0.9279871,  0.8512805, 1.0449820,  0.9670631, &
    0.9857884,  0.9676313, 1.0129299,  0.9657040, &
    1.0019855,  1.0359297, 1.0414677,  0.9471506  /)
REAL (dp), PARAMETER  :: y1(1:100) = (/   &
   -0.0310206,  0.1586742,  0.2576924, 0.3414014, &
    0.4943596,  0.5782854,  0.6993418, 0.7470194, &
    0.9107649,  0.9962890,  0.0501330, 0.0918555, &
    0.2592973,  0.3381592,  0.4171125, 0.5615563, &
    0.6552235,  0.7524066,  0.9146523, 0.9632421, &
    0.0292939,  0.0602303,  0.2668783, 0.3696044, &
    0.4801738,  0.5940595,  0.6878797, 0.8185576, &
    0.9046507,  0.9805412,  0.0396955, 0.0684484, &
    0.2389548,  0.3124129,  0.4902989, 0.5199303, &
    0.6445227,  0.8203789,  0.8938079, 0.9711719, &
   -0.0284618,  0.1560965,  0.2262471, 0.3175094, &
    0.3891417,  0.5084949,  0.6324247, 0.7511007, &
    0.8489712,  0.9978728, -0.0271948, 0.1272430, &
    0.2709269,  0.3477728,  0.4259422, 0.6084711, &
    0.6733781,  0.7235242,  0.9242411, 1.0308762, &
    0.0255959,  0.0707835,  0.2008336, 0.3259843, &
    0.4890704,  0.5096324,  0.6697880, 0.7759569, &
    0.9366096,  1.0064516,  0.0285374, 0.1021403, &
    0.1936581,  0.3235775,  0.4714228, 0.6091595, &
    0.6685053,  0.8022808,  0.8476790, 1.0512371, &
    0.0380499,  0.0902048,  0.2083092, 0.3318491, &
    0.4335632,  0.5910139,  0.6307383, 0.8144841, &
    0.9042310,  0.9696030, -0.0120900, 0.1334114, &
    0.2695844,  0.3795281,  0.4396054, 0.5044425, &
    0.6941519,  0.7459923,  0.8682081, 0.9801409 /)

! Node set 2:  Franke's 33-node set.

REAL (dp), PARAMETER  :: x2(1:33) = (/   &
    0.05, 0.00, 0.00, 0.00, 0.10, 0.10, 0.15, 0.20,  &
    0.25, 0.30, 0.35, 0.50, 0.50, 0.55, 0.60, 0.60,  &
    0.60, 0.65, 0.70, 0.70, 0.70, 0.75, 0.75, 0.75,  &
    0.80, 0.80, 0.85, 0.90, 0.90, 0.95, 1.00, 1.00, 1.00 /)
REAL (dp), PARAMETER  :: y2(1:33) = (/   &
    0.45, 0.50, 1.00, 0.00, 0.15, 0.75, 0.30, 0.10,  &
    0.20, 0.35, 0.85, 0.00, 1.00, 0.95, 0.25, 0.65,  &
    0.85, 0.70, 0.20, 0.65, 0.90, 0.10, 0.35, 0.85,  &
    0.40, 0.65, 0.25, 0.35, 0.80, 0.90, 0.00, 0.50, 1.00 /)

! Node set 3:  Lawson's 25-node set.

REAL (dp), PARAMETER  :: x3(1:25) = (/   &
    0.13750,  0.91250,  0.71250,  0.22500, &
   -0.05000,  0.47500,  0.05000,  0.45000, &
    1.08750,  0.53750, -0.03750,  0.18750, &
    0.71250,  0.85000,  0.70000,  0.27500, &
    0.45000,  0.81250,  0.45000,  1.00000, &
    0.50000,  0.18750,  0.58750,  1.05000,  0.10000 /)
REAL (dp), PARAMETER  :: y3(1:25) = (/   &
    0.97500,  0.98750,  0.76250,  0.83750, &
    0.41250,  0.63750, -0.05000,  1.03750, &
    0.55000,  0.80000,  0.75000,  0.57500, &
    0.55000,  0.43750,  0.31250,  0.42500, &
    0.28750,  0.18750, -0.03750,  0.26250, &
    0.46250,  0.26250,  0.12500, -0.06125,  0.11250 /)

! Node set 4:  Random 100-node set.

REAL (dp), PARAMETER  :: x4(1:100) = (/   &
    0.0096326, 0.0216348, 0.0298360, 0.0417447, &
    0.0470462, 0.0562965, 0.0646857, 0.0740377, &
    0.0873907, 0.0934832, 0.1032216, 0.1110176, &
    0.1181193, 0.1251704, 0.1327330, 0.1439536, &
    0.1564861, 0.1651043, 0.1786039, 0.1886405, &
    0.2016706, 0.2099886, 0.2147003, 0.2204141, &
    0.2343715, 0.2409660, 0.2527740, 0.2570839, &
    0.2733365, 0.2853833, 0.2901755, 0.2964854, &
    0.3019725, 0.3125695, 0.3307163, 0.3378504, &
    0.3439061, 0.3529922, 0.3635507, 0.3766172, &
    0.3822429, 0.3869838, 0.3973137, 0.4170708, &
    0.4255588, 0.4299218, 0.4372839, 0.4705033, &
    0.4736655, 0.4879299, 0.4940260, 0.5055324, &
    0.5162593, 0.5219219, 0.5348529, 0.5483213, &
    0.5569571, 0.5638611, 0.5784908, 0.5863950, &
    0.5929148, 0.5987839, 0.6117561, 0.6252296, &
    0.6331381, 0.6399048, 0.6488972, 0.6558537, &
    0.6677405, 0.6814074, 0.6887812, 0.6940896, &
    0.7061687, 0.7160957, 0.7317445, 0.7370798, &
    0.7462030, 0.7566957, 0.7699998, 0.7879347, &
    0.7944014, 0.8164468, 0.8192794, 0.8368405, &
    0.8500993, 0.8588255, 0.8646496, 0.8792329, &
    0.8837536, 0.8900077, 0.8969894, 0.9044917, &
    0.9083947, 0.9203972, 0.9347906, 0.9434519, &
    0.9490328, 0.9569571, 0.9772067, 0.9983493 /)
REAL (dp), PARAMETER  :: y4(1:100) = (/   &
    0.3083158, 0.2450434, 0.8613847, 0.0977864, &
    0.3648355, 0.7156339, 0.5311312, 0.9755672, &
    0.1781117, 0.5452797, 0.1603881, 0.7837139, &
    0.9982015, 0.6910589, 0.1049580, 0.8184662, &
    0.7086405, 0.4456593, 0.1178342, 0.3189021, &
    0.9668446, 0.7571834, 0.2016598, 0.3232444, &
    0.4368583, 0.8907869, 0.0647260, 0.5692618, &
    0.2947027, 0.4332426, 0.3347464, 0.7436284, &
    0.1066265, 0.8845357, 0.5158730, 0.9425637, &
    0.4799701, 0.1783069, 0.1146760, 0.8225797, &
    0.2270688, 0.4073598, 0.8875080, 0.7631616, &
    0.9972804, 0.4959884, 0.3410421, 0.2498120, &
    0.6409007, 0.1058690, 0.5411969, 0.0089792, &
    0.8784268, 0.5515874, 0.4038952, 0.1654023, &
    0.2965158, 0.3660356, 0.0366554, 0.9502420, &
    0.2638101, 0.9277386, 0.5377694, 0.7374676, &
    0.4674627, 0.9186109, 0.0416884, 0.1291029, &
    0.6763676, 0.8444238, 0.3273328, 0.1893879, &
    0.0645923, 0.0180147, 0.8904992, 0.4160648, &
    0.4688995, 0.2174508, 0.5734231, 0.8853319, &
    0.8018436, 0.6388941, 0.8931002, 0.1000558, &
    0.2789506, 0.9082948, 0.3259159, 0.8318747, &
    0.0508513, 0.9708450, 0.5120548, 0.2859716, &
    0.9581641, 0.6183429, 0.3779934, 0.4010423, &
    0.9478657, 0.7425486, 0.8883287, 0.5496750 /)

! Node set 5:  9 by 9 uniform grid.

REAL (dp), PARAMETER  :: x5(1:81) = (/   &
    0.125, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, &
    0.000, 0.000, 0.125, 0.125, 0.125, 0.125, 0.125, 0.125, &
    0.125, 0.125, 0.250, 0.250, 0.250, 0.250, 0.250, 0.250, &
    0.250, 0.250, 0.250, 0.375, 0.375, 0.375, 0.375, 0.375, &
    0.375, 0.375, 0.375, 0.375, 0.500, 0.500, 0.500, 0.500, &
    0.500, 0.500, 0.500, 0.500, 0.500, 0.625, 0.625, 0.625, &
    0.625, 0.625, 0.625, 0.625, 0.625, 0.625, 0.750, 0.750, &
    0.750, 0.750, 0.750, 0.750, 0.750, 0.750, 0.750, 0.875, &
    0.875, 0.875, 0.875, 0.875, 0.875, 0.875, 0.875, 0.875, &
    1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, &
    1.000 /)
REAL (dp), PARAMETER  :: y5(1:81) = (/   &
    0.000, 0.125, 0.250, 0.375, 0.500, 0.625, 0.750, 0.875, &
    1.000, 0.000, 0.125, 0.250, 0.375, 0.500, 0.625, 0.750, &
    0.875, 1.000, 0.000, 0.125, 0.250, 0.375, 0.500, 0.625, &
    0.750, 0.875, 1.000, 0.000, 0.125, 0.250, 0.375, 0.500, &
    0.625, 0.750, 0.875, 1.000, 0.000, 0.125, 0.250, 0.375, &
    0.500, 0.625, 0.750, 0.875, 1.000, 0.000, 0.125, 0.250, &
    0.375, 0.500, 0.625, 0.750, 0.875, 1.000, 0.000, 0.125, &
    0.250, 0.375, 0.500, 0.625, 0.750, 0.875, 1.000, 0.000, &
    0.125, 0.250, 0.375, 0.500, 0.625, 0.750, 0.875, 1.000, &
    0.000, 0.125, 0.250, 0.375, 0.500, 0.625, 0.750, 0.875, &
    1.000 /)

! Store node set K in (X,Y).

IF (k == 1) THEN
  DO  i = 1,100
    x(i) = x1(i)
    y(i) = y1(i)
  END DO
  n = 100
ELSE IF (k == 2) THEN
  DO  i = 1,33
    x(i) = x2(i)
    y(i) = y2(i)
  END DO
  n = 33
ELSE IF (k == 3) THEN
  DO  i = 1,25
    x(i) = x3(i)
    y(i) = y3(i)
  END DO
  n = 25
ELSE IF (k == 4) THEN
  DO  i = 1,100
    x(i) = x4(i)
    y(i) = y4(i)
  END DO
  n = 100
ELSE IF (k == 5) THEN
  DO  i = 1,81
    x(i) = x5(i)
    y(i) = y5(i)
  END DO
  n = 81
ELSE
  n = 0
END IF

RETURN
END SUBROUTINE testdt


SUBROUTINE tstfn1 (k, x, y, iflag, f, fx, fy)

INTEGER, INTENT(IN)     :: k
REAL (dp), INTENT(IN)   :: x
REAL (dp), INTENT(IN)   :: y
INTEGER, INTENT(IN)     :: iflag
REAL (dp), INTENT(OUT)  :: f
REAL (dp), INTENT(OUT)  :: fx
REAL (dp), INTENT(OUT)  :: fy

!***********************************************************

!                                            Robert J. Renka
!                                  Dept. of Computer Science
!                                       Univ. of North Texas
!                                           renka@cs.unt.edu
!                                                   10/14/98

!   This subroutine computes the value and, optionally, the
! first partial derivatives of one of ten bivariate test
! functions.  The first six functions were chosen by Richard
! Franke to test interpolation software (See the reference
! below).  The last four functions represent more challenging
! surface fitting problems.

! On input:

!       K = Integer in the range 1 to 10 which determines
!           the choice of function as follows:

!               K = 1 - Exponential
!               K = 2 - Cliff
!               K = 3 - Saddle
!               K = 4 - Gentle
!               K = 5 - Steep
!               K = 6 - Sphere
!               K = 7 - Trig
!               K = 8 - Synergistic Gaussian
!               K = 9 - Cloverleaf Asymmetric Peak/Valley
!               K = 10 - Cosine Peak

!   Note that function 6 is only defined inside a circle of
! radius 8/9 centered at (.5,.5).  Thus, if (X-.5)**2 +
! (Y-.5)**2 .GE. 64/81, the value (and partials if IFLAG=1)
! are set to 0 for this function.  Also, the first partial
! derivatives of function 10 are not defined at (.5,.5) --
! again, zeros are returned.

!       X,Y = Coordinates of the point at which the selected
!             function is to be evaluated.

!       IFLAG = Derivative option indicator:
!               IFLAG = 0 if only a function value is required.
!               IFLAG = 1 if both the function and its first
!                         partial derivatives are to be evaluated.

! Input parameters are not altered by this routine.

! On output:

!       F = Value of function K at (X,Y).

!       FX,FY = First partial derivatives of function K at
!               (X,Y) if IFLAG = 1, unaltered otherwise.

! Intrinsic functions called by TSTFN1:  COS, EXP, SIN, SQRT, TANH

! Reference:  R. Franke, A Critical Comparison of Some
!               Methods for Interpolation of Scattered Data,
!               Naval Postgraduate School Technical Report,
!               NPS-53-79-003, 1979.

!***********************************************************

REAL (dp) :: t1, t2, t3, t4

IF (k < 1  .OR.  k > 10) RETURN
SELECT CASE ( k )

  CASE (    1)  ! Exponential:

    f = .75*EXP(-((9.*x-2.)**2 + (9.*y-2.)**2)/4.) +  &
        .75*EXP(-((9.*x+1.)**2)/49. - (9.*y+1.)/10.) +  &
        .5*EXP(-((9.*x-7.)**2 + (9.*y-3.)**2)/4.) -  &
        .2*EXP(-(9.*x-4.)**2 - (9.*y-7.)**2)
    IF (iflag /= 1) RETURN
    t1 = EXP(-((9.*x-2.)**2 + (9.*y-2.)**2)/4.)
    t2 = EXP(-((9.*x+1.)**2)/49. - (9.*y+1.)/10.)
    t3 = EXP(-((9.*x-7.)**2 + (9.*y-3.)**2)/4.)
    t4 = EXP(-(9.*x-4.)**2 - (9.*y-7.)**2)
    fx = -3.375*(9.*x-2.)*t1 - (27./98.)*(9.*x+1.)*t2  &
        -2.25*(9.*x-7.)*t3 + 3.6*(9.*x-4.)*t4
    fy = -3.375*(9.*y-2.)*t1 - .675*t2 -2.25*(9.*y-3.)*t3 + 3.6*(9.*y-7.)*t4
    RETURN

  CASE (    2)  ! Cliff:

    f = (TANH(9.0*(y-x)) + 1.0)/9.0
    IF (iflag /= 1) RETURN
    t1 = 18.0*(y-x)
    fx = -4.0/(EXP(t1) + 2.0 + EXP(-t1))
    fy = -fx
    RETURN

  CASE (    3)  ! Saddle:

    f = (1.25 + COS(5.4_dp*y))/(6.0 + 6.0*(3.0*x-1.0)**2)
    IF (iflag /= 1) RETURN
    t1 = 5.4_dp*y
    t2 = 1.0 + (3.0*x-1.)**2
    fx = -(3.0*x-1.0)*(1.25 + COS(t1))/(t2**2)
    fy = -0.9_dp*SIN(t1)/t2
    RETURN

  CASE (    4)  ! Gentle:

    f = EXP(-5.0625*((x-.5)**2 + (y-.5)**2))/3.0
    IF (iflag /= 1) RETURN
    t1 = x - .5
    t2 = y - .5
    t3 = -3.375*EXP(-5.0625*(t1**2 + t2**2))
    fx = t1*t3
    fy = t2*t3
    RETURN

  CASE (    5)
    ! Steep:

    f = EXP(-20.25*((x-.5)**2 + (y-.5)**2))/3.0
    IF (iflag /= 1) RETURN
    t1 = x - .5
    t2 = y - .5
    t3 = -13.5*EXP(-20.25*(t1**2 + t2**2))
    fx = t1*t3
    fy = t2*t3
    RETURN

  CASE (    6)  ! Sphere:

    t4 = 64.0 - 81.0*((x-.5)**2 + (y-.5)**2)
    f = 0.
    IF (t4 >= 0.) f = SQRT(t4)/9.0 - .5
    IF (iflag /= 1) RETURN
    t1 = x - .5
    t2 = y - .5
    t3 = 0.
    IF (t4 > 0.) t3 = -9.0/SQRT(t4)
    fx = t1*t3
    fy = t2*t3
    RETURN

  CASE (    7)  ! Trig:

    f = 2.0*COS(10.0*x)*SIN(10.0*y) + SIN(10.0*x*y)
    IF (iflag /= 1) RETURN
    t1 = 10.0*x
    t2 = 10.0*y
    t3 = 10.0*COS(10.0*x*y)
    fx = -20.0*SIN(t1)*SIN(t2) + t3*y
    fy = 20.0*COS(t1)*COS(t2) + t3*x
    RETURN

  CASE (    8)
    ! Gaussx(1,.5,.1) + Gaussy(.75,.5,.1) + Gaussx(1,.5,.1)*Gaussy(.75,.5,.1),
    !   where Gaussx(a,b,c) is the Gaussian function of x with amplitude a,
    !   center (mean) b, and width (standard deviation) c.

    t1 = 5.0 - 10.0*x
    t2 = 5.0 - 10.0*y
    t3 = EXP(-.5*t1*t1)
    t4 = EXP(-.5*t2*t2)
    f = t3 + .75*t4*(1.0+t3)
    IF (iflag /= 1) RETURN
    fx = t1*t3*(10.0 + 7.5*t4)
    fy = t2*t4*(7.5 + 7.5*t3)
    RETURN

  CASE (    9)  ! Cloverleaf Asymmetric Hill/Valley:

    t1 = EXP((10.0 - 20.0*x)/3.0)
    t2 = EXP((10.0 - 20.0*y)/3.0)
    t3 = 1.0/(1.0 + t1)
    t4 = 1.0/(1.0 + t2)
    f = ((20.0/3.0)**3 * t1*t2)**2 * (t3*t4)**5 * (t1-2.0*t3)*(t2-2.0*t4)
    IF (iflag /= 1) RETURN
    fx = ((20.0/3.0)*t1)**2 * ((20.0/3.0)*t3)**5 *  &
         (2.0*t1-3.0*t3-5.0+12.0*t3*t3)*t2*t2*t4**5 * (t2-2.0*t4)
    fy = ((20.0/3.0)*t1)**2 * ((20.0/3.0)*t3)**5 *  &
         (2.0*t2-3.0*t4-5.0+12.0*t4*t4)*t2*t2*t4**5 * (t1-2.0*t3)
    RETURN

  CASE (   10)  ! Cosine Peak:

    t1 = SQRT( (80.0*x - 40.0)**2 + (90.0*y - 45.0)**2 )
    t2 = EXP(-.04_dp*t1)
    t3 = COS(.15_dp*t1)
    f = t2*t3
    IF (iflag /= 1) RETURN
    t4 = SIN(.15_dp*t1)
    fx = 0.
    fy = 0.
    IF (t1 == 0.) RETURN
    t4 = SIN(.15_dp*t1)
    fx = -t2*(12.0*t4 + 3.2_dp*t3)*(80.0*x - 40.0)/t1
    fy = -t2*(13.5*t4 + 3.6_dp*t3)*(90.0*y - 45.0)/t1
    RETURN

END SELECT

RETURN
END SUBROUTINE tstfn1

END PROGRAM driver1
