PROGRAM driver2
USE Cubic_Shepard
IMPLICIT NONE
 
! Code converted using TO_F90 by Alan Miller
! Date: 1999-08-10  Time: 20:55:52
 
!                          CS2TEST
!                          02/19/97

!   This program tests the scattered data interpolation package CSHEP2D
! by printing the maximum errors associated with interpolated values and
! gradients on a 20 by 20 uniform grid in the unit square.
! The data set consists of 100 nodes with data values taken from a cubic
! function for which the method is exact.  The ratio of maximum interpolation
! error relative to the machine precision is also printed.  This should be
! O(1).  The interpolated values from CS2VAL, CS2GRD, and CS2HES are compared
! for agreement.

INTEGER, PARAMETER  :: n=100, nr=6

! Array storage:

INTEGER    :: lcell(nr,nr), lnext(n)
REAL (dp)  :: x(n), y(n), f(n), rw(n), a(9,n), p(20)

INTEGER    :: i, ier, j, k
REAL (dp)  :: c1, c2, c3, cx, cy, cxx, cxy, cyy, dx, dy, ec, ecx, ecy, ecxx, &
              ecxy, ecyy, ep1, eps, px, py, rc, rmax, xmin, ymin, yk, xx, yy

! CSHEP2 parameters and logical unit for output:

INTEGER, PARAMETER  :: nc = 17,  nw = 30,  lout = 1

! Output file:

OPEN (lout, FILE='res2')

! Generate a 10 by 10 grid of nodes in the unit square with
! the natural ordering.

k = 0
DO  j = 1,10
  yk = DBLE(10-j)/9.
  DO  i = 1,10
    k = k + 1
    x(k) = DBLE(i-1)/9.
    y(k) = yk
  END DO
END DO

! Compute the data values.

DO  k = 1,n
  f(k) = fc(x(k),y(k))
END DO

! Compute parameters defining the interpolant C.

CALL cshep2 (n, x, y, f, nc, nw, nr, lcell, lnext, xmin, ymin, dx, dy, rmax, &
             rw, a, ier)
IF (ier /= 0) GO TO 10

! Generate a 20 by 20 uniform grid of interpolation points (P(I),P(J))
!   in the unit square.  The four corners coincide with nodes.

DO  i = 1,20
  p(i) = DBLE(i-1)/19.
END DO

! Compute the machine precision EPS.

eps = 1.
5 eps = eps/2.
ep1 = eps + 1.
IF (store(ep1) > 1.) GO TO 5
eps = eps*2.

! Compute interpolation errors and test for agreement in the
!   C values returned by CS2VAL, CS2GRD, and CS2HES.

ec = 0.
ecx = 0.
ecy = 0.
ecxx = 0.
ecxy = 0.
ecyy = 0.
DO  j = 1,20
  py = p(j)
  DO  i = 1,20
    px = p(i)
    c1 = cs2val (px, py, n, x, y, f, nr, lcell, lnext, xmin, ymin, dx, dy,  &
                 rmax, rw, a)
    CALL cs2grd (px, py, n, x, y, f, nr, lcell, lnext, xmin, ymin, dx, dy,  &
                 rmax, rw, a,  c2, cx, cy, ier)
    CALL cs2hes (px, py, n, x, y, f, nr, lcell, lnext, xmin, ymin, dx, dy,  &
                 rmax, rw, a, c3, cx, cy, cxx,  cxy, cyy, ier)
    IF (ier /= 0) GO TO 11
    IF (ABS(c1-c2) > 3.*ABS(c1)*eps .OR. ABS(c1-c3) > 3.*ABS(c1)*eps) THEN
      
! Values returned by CS2VAL, CS2GRD, and CS2HES differ by a
!   relative amount greater than 3*EPS.
      
      WRITE (lout,90) c1, c2, c3
      90 FORMAT (//' *** Error -- interpolated values C1 (CS2VAL), C2 (CS2GRD), and'/ &
                 t6, 'C3 (CS2HES) differ:'//  &
                 t6, 'C1 = ', g21.14/ t6, 'C2 = ', g21.14/ t6, 'C3 = ', g21.14)
    END IF
    ec = MAX(ec, ABS(fc(px,py)-c1))
    ecx = MAX(ecx, ABS(fx(px,py)-cx))
    ecy = MAX(ecy, ABS(fy(px,py)-cy))
    ecxx = MAX(ecxx, ABS(fxx(px,py)-cxx))
    ecxy = MAX(ecxy, ABS(fxy(px,py)-cxy))
    ecyy = MAX(ecyy, ABS(fyy(px,py)-cyy))
  END DO
END DO

! Print errors and the ratio EC/EPS.

rc = ec/eps
WRITE (lout,100)
WRITE (lout,110) ec, rc
WRITE (lout,120) ecx
WRITE (lout,130) ecy
WRITE (lout,140) ecxx
WRITE (lout,150) ecxy
WRITE (lout,160) ecyy
STOP
100 FORMAT (//' Maximum absolute errors in the interpolant C and partial'/  &
            ' derivatives CX, ..., CYY relative to machine precision EPS'//  &
            t12, 'Function   Max Error   Max Error/EPS'/)
110 FORMAT (t15, 'C       ', e9.3, '       ', f4.2)
120 FORMAT (t15, 'CX      ', e9.3)
130 FORMAT (t15, 'CY      ', e9.3)
140 FORMAT (t15, 'CXX     ', e9.3)
150 FORMAT (t15, 'CXY     ', e9.3)
160 FORMAT (t15, 'CYY     ', e9.3)

! Error in CSHEP2.

10 WRITE (lout,200) ier
STOP
200 FORMAT (//' *** Error in CSHEP2 -- IER =', i2, ' ***')

! Error in CS2GRD.

11 WRITE (lout,210) ier
STOP
210 FORMAT (//' *** Error in CS2GRD -- IER =', i2, ' ***')


CONTAINS


FUNCTION store (x) RESULT(fn_val)

REAL (dp), INTENT(IN)  :: x
REAL (dp)              :: fn_val

!***********************************************************

!                                               From TRIPACK
!                                            Robert J. Renka
!                                  Dept. of Computer Science
!                                       Univ. of North Texas
!                                           renka@cs.unt.edu
!                                                   03/18/90

!   This function forces its argument X to be stored in a
! memory location, thus providing a means of determining
! floating point number characteristics (such as the machine
! precision) when it is necessary to avoid computation in
! high precision registers.


! On input:

!       X = Value to be stored.

! X is not altered by this function.

! On output:

!       STORE = Value of X after it has been stored and
!               possibly truncated or rounded to the single
!               precision word length.

! Modules required by STORE:  None

!***********************************************************

REAL (dp), SAVE  :: y
! COMMON/stcom/y

y = x
fn_val = y
RETURN
END FUNCTION store



! Cubic test function and partial derivatives:

FUNCTION fc(xx, yy) RESULT(fn_val)
REAL (dp), INTENT(IN)  :: xx, yy
REAL (dp)              :: fn_val

fn_val = ((xx + 2.*yy)/3.)**3
RETURN
END FUNCTION fc



FUNCTION fx(xx, yy) RESULT(fn_val)
REAL (dp), INTENT(IN)  :: xx, yy
REAL (dp)              :: fn_val

fn_val = ((xx + 2.*yy)/3.)**2
RETURN
END FUNCTION fx



FUNCTION fy(xx, yy) RESULT(fn_val)
REAL (dp), INTENT(IN)  :: xx, yy
REAL (dp)              :: fn_val

fn_val = 2.*((xx + 2.*yy)/3.)**2
RETURN
END FUNCTION fy



FUNCTION fxx(xx, yy) RESULT(fn_val)
REAL (dp), INTENT(IN)  :: xx, yy
REAL (dp)              :: fn_val

fn_val = 2.*(xx + 2.*yy)/9.
RETURN
END FUNCTION fxx



FUNCTION fxy(xx, yy) RESULT(fn_val)
REAL (dp), INTENT(IN)  :: xx, yy
REAL (dp)              :: fn_val

fn_val = 4.*(xx + 2.*yy)/9.
RETURN
END FUNCTION fxy



FUNCTION fyy(xx, yy) RESULT(fn_val)
REAL (dp), INTENT(IN)  :: xx, yy
REAL (dp)              :: fn_val

fn_val = 8.*(xx + 2.*yy)/9.
RETURN
END FUNCTION fyy

END PROGRAM driver2

